#include "stdafx.h"

#include <msgqueue.h>
#include <pm.h>

#include "PowerWatch.h"

#define MAX_MESSAGES		5
#define MAX_MESSAGE_SIZE	(sizeof(POWER_BROADCAST) + MAX_PATH)


DWORD
WINAPI
WatchProc(
  LPVOID pParam
)
{
	CPowerWatch*	pPowerWatch = NULL;
	HANDLE			hEvents[2];
	DWORD			dwObject;


	pPowerWatch = (CPowerWatch*)pParam;

	if (pPowerWatch != NULL) {

		hEvents[0] = pPowerWatch->m_hStopEvent;
		hEvents[1] = pPowerWatch->m_hMsgQueue;

		while (!pPowerWatch->m_bStopRequest) {

			dwObject = WaitForMultipleObjects(2, hEvents, FALSE, INFINITE);
			switch(dwObject) {
			case WAIT_OBJECT_0:
				// Iʒm
				;
				break;
			case WAIT_OBJECT_0 + 1:
				// Queueǂݍ݉\
				{
					BOOL	bRead		= FALSE;
					DWORD	dwReaded	= 0;
					DWORD	dwFlags		= 0;

					memset(pPowerWatch->m_pBuffer, 0, pPowerWatch->m_nBufferSize);
					bRead = ReadMsgQueue(	pPowerWatch->m_hMsgQueue,
											pPowerWatch->m_pBuffer,
											pPowerWatch->m_nBufferSize,
											&dwReaded,
											MAX_MESSAGES,
											&dwFlags	);
					if (bRead) {
						if (dwReaded >= sizeof(POWER_BROADCAST)) {
							POWER_BROADCAST* pb = (POWER_BROADCAST*)pPowerWatch->m_pBuffer;
							if (pb->Message & PBT_RESUME) {
								if (pPowerWatch->m_pfnNotifyResume != NULL) {
									(*(pPowerWatch->m_pfnNotifyResume))(pPowerWatch->m_pUserData);
								}
							}
						}
					}

				}
				break;
			}

		}
	}

	return 0;
}

//////////////////////////////////////////////////////////////////////
// \z/
//////////////////////////////////////////////////////////////////////

CPowerWatch::CPowerWatch()
{
	m_bStopRequest	= NULL;
	m_hStopEvent	= NULL;
	m_hThread		= NULL;
	m_hMsgQueue		= NULL;
	m_hNotify		= NULL;
	m_nBufferSize	= 0;
	m_pBuffer		= NULL;

	m_pfnNotifyResume	= NULL;
	m_pUserData			= NULL;
}

CPowerWatch::~CPowerWatch()
{
	if (m_hThread != NULL) {
		StopWatch();
	}
}

BOOL
CPowerWatch::StartWatch(PFNNOTIFYRESUME pfnNotifyResume, void* pUserData)
{
	BOOL				bRet;
	DWORD				dwThreadId;
	MSGQUEUEOPTIONS		msgOptions;

	bRet = FALSE;

	if (m_hThread != NULL) {
		return FALSE;
	}


	m_nBufferSize = MAX_MESSAGE_SIZE*MAX_MESSAGES;
	m_pBuffer = (LPBYTE)malloc(m_nBufferSize);

	if (m_pBuffer != NULL) {

		m_hStopEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
		if (m_hStopEvent != NULL) {

			memset(&msgOptions, 0, sizeof(msgOptions));
			msgOptions.dwSize = sizeof(msgOptions);
			msgOptions.dwFlags = 0;
			msgOptions.dwMaxMessages = MAX_MESSAGES;
			msgOptions.cbMaxMessage = MAX_MESSAGE_SIZE;
			msgOptions.bReadAccess = TRUE;

			m_hMsgQueue = CreateMsgQueue(NULL, &msgOptions);
			if (m_hMsgQueue != NULL) {
				m_hNotify = RequestPowerNotifications(m_hMsgQueue, PBT_RESUME);
				if (m_hNotify != NULL) {

					m_bStopRequest = FALSE;
					m_pfnNotifyResume = pfnNotifyResume;
					m_pUserData = pUserData;

					m_hThread = CreateThread(NULL, 0, WatchProc, this, 0, &dwThreadId);
			
					if (m_hThread != NULL) {
						bRet = TRUE;
					}
				}
			}
		}
	}

	if (!bRet) {
		if (m_hNotify != NULL) {
			StopPowerNotifications(m_hNotify);
			m_hNotify = NULL;
		}
		if (m_hMsgQueue != NULL) {
			CloseMsgQueue(m_hMsgQueue);
			m_hMsgQueue = NULL;
		}
		if (m_hStopEvent != NULL) {
			CloseHandle(m_hStopEvent);
			m_hStopEvent = NULL;
		}
		if (m_pBuffer != NULL) {
			free(m_pBuffer);
			m_pBuffer = NULL;

			m_nBufferSize = 0;
		}

		m_pfnNotifyResume	= NULL;
		m_pUserData			= NULL;
	}

	return bRet;
}

BOOL
CPowerWatch::StopWatch()
{
	if (m_hThread != NULL) {
		m_bStopRequest = TRUE;
		
		if (m_hStopEvent != NULL) {
			SetEvent(m_hStopEvent);
		}

		WaitForSingleObject(m_hThread, INFINITE);
		CloseHandle(m_hThread);
		m_hThread = NULL;

		m_bStopRequest = FALSE;
	}

	if (m_hNotify != NULL) {
		StopPowerNotifications(m_hNotify);
		m_hNotify = NULL;
	}
	if (m_hMsgQueue != NULL) {
		CloseMsgQueue(m_hMsgQueue);
		m_hMsgQueue = NULL;
	}
	if (m_hStopEvent != NULL) {
		CloseHandle(m_hStopEvent);
		m_hStopEvent = NULL;
	}
	if (m_pBuffer != NULL) {
		free(m_pBuffer);
		m_pBuffer = NULL;

		m_nBufferSize = 0;
	}
	m_pfnNotifyResume	= NULL;
	m_pUserData			= NULL;

	return TRUE;
}

